import {
  EthFlowStepper as Pure,
  EthFlowStepperProps as PureProps,
  SmartOrderStatus,
} from '@cow/modules/swap/pure/EthFlow/EthFlowStepper'
import { useDetectNativeToken } from '@cow/modules/swap/hooks/useDetectNativeToken'
import { Order, OrderStatus } from 'state/orders/actions'
import { NATIVE_CURRENCY_BUY_ADDRESS } from 'constants/index'
import { safeTokenName } from '@cowprotocol/cow-js'
import { isOrderExpired } from 'state/orders/utils'
import { useAllTransactions } from 'state/enhancedTransactions/hooks'
import { EnhancedTransactionDetails } from 'state/enhancedTransactions/reducer'

type EthFlowStepperProps = {
  order: Order | undefined
}

export function EthFlowStepper(props: EthFlowStepperProps) {
  const { order } = props
  const { native } = useDetectNativeToken()

  const allTxs = useAllTransactions()

  const creationHash = order?.orderCreationHash
  const cancellationHash = order?.cancellationHash
  // TODO: add refund hash when available from API

  const creationTx = creationHash ? allTxs[creationHash] : undefined
  const cancellationTx = cancellationHash ? allTxs[cancellationHash] : undefined

  const state = mapOrderToEthFlowStepperState(order, creationTx, cancellationTx)

  const isEthFlowOrder = getIsEthFlowOrder(order)

  if (!order || !state || !isEthFlowOrder) {
    return null
  }

  const stepperProps: PureProps = {
    nativeTokenSymbol: native.symbol as string,
    tokenLabel: safeTokenName(order.outputToken),
    order: {
      // The creation hash is only available in the device where the order is placed
      createOrderTx: creationHash || '',
      orderId: order.id,
      state,
      isExpired: isEthFlowOrderExpired(order),
      isCreated: !!order.apiAdditionalInfo,
      // rejectedReason?: TODO: address when dealing with rejections
    },
    // TODO: fill these in when dealing with rejections
    refund: {
      // refundTx?: string
      isRefunded: false,
    },
    cancellation: {
      cancellationTx: cancellationHash,
      isCancelled: isEthFlowOrderCancelled(order, cancellationTx),
    },
  }

  return <Pure {...stepperProps} />
}

const ORDER_INDEXED_STATUSES: OrderStatus[] = [OrderStatus.PENDING, OrderStatus.EXPIRED, OrderStatus.CANCELLED]

function mapOrderToEthFlowStepperState(
  order: Order | undefined,
  creationTx: EnhancedTransactionDetails | undefined,
  cancellationTx: EnhancedTransactionDetails | undefined
): SmartOrderStatus | undefined {
  if (order) {
    const { status } = order

    if (status === 'fulfilled') {
      return SmartOrderStatus.FILLED
    } else if (ORDER_INDEXED_STATUSES.includes(status) || cancellationTx?.receipt) {
      return SmartOrderStatus.INDEXED
    } else if (status === 'creating') {
      if (creationTx?.receipt) {
        return SmartOrderStatus.CREATION_MINED
      }
      return SmartOrderStatus.CREATING
    }
  }
  return undefined
}

function isEthFlowOrderExpired(order: Order | undefined): boolean {
  return order?.status === 'expired' || isOrderExpired({ validTo: order?.validTo as number })
}

function isEthFlowOrderCancelled(order: Order, cancellationTx: EnhancedTransactionDetails | undefined): boolean {
  return order.status === 'cancelled' || !!cancellationTx?.receipt
}

// TODO: move this somewhere else?
export function getIsEthFlowOrder(order: Order | undefined): boolean {
  return order?.inputToken.address === NATIVE_CURRENCY_BUY_ADDRESS
}
